﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BSSI.APLWinNet35
{
    /// <summary>
    /// C# .Net 3.5 interface to APL+Win as an ActiveX server
    /// Using this interface class means that the .Net calling environment project does not require a reference to APL+Win (APLW)
    /// This project implements APLNotify and APLSysNotify events which use  the .Net standard event handler signature
    /// i.e. event_handler_name(object sender, ...EventArgs e) rather 
    /// In addition this project provides the opportunity to implement 'friendly' APL+Win method signatures
    /// and overloaded methods which eliminate the need for 'missing' arguments
    /// For proper operation, APL+Win must be registered as an ActiveX server on the target machine
    /// </summary>
    public class APLWinInterface
        //^The APLWinInterface class should not inherit from APLW.WSEngine
    {
        private APLW.WSEngine wsEngine;
        public APLWinInterface()
        {
            wsEngine = new APLW.WSEngine();
            wsEngine.Notify += new APLW._WSEngineEvents_NotifyEventHandler(wsEngine_Notify);
            wsEngine.SysNotify += new APLW._WSEngineEvents_SysNotifyEventHandler(wsEngine_SysNotify);
        }
        public delegate void APLNotifyHandler(object sender, APLNotifyEventArgs e);
        public event APLNotifyHandler APLNotify;
        void wsEngine_Notify(ref object Code, ref object Data, out object Result)
        {
            Result = "Default response back to APL+Win: Notify event handled by BSSI.APLWinNet35";
            if (this.APLNotify != null)
            {
                APLNotifyEventArgs e = new APLNotifyEventArgs(Code, Data, Result);
                this.APLNotify(this, e);
                Result = e.Result;
                //^ Send e.Result from the .Net standard signature event handler back to
                //  APL+Win via the "out object Result" argument of the APL+Win native event handler
            }
        }
        public delegate void APLSysNotifyHandler(object sender, APLSysNotifyEventArgs e);
        public event APLSysNotifyHandler APLSysNotify;
        void wsEngine_SysNotify(ref object Code, ref object Data, out object Result)
        {
            Result = "Default response back to APL+Win: SysNotify event handled by BSSI.APLWinNet35";
            if (this.APLSysNotify != null)
            {
                APLSysNotifyEventArgs e = new APLSysNotifyEventArgs(Code, Data, Result);
                this.APLSysNotify(this, e);
                Result = e.Result;
                //^ Send e.Result from the .Net standard signature event handler back to
                //  APL+Win via the "out object Result" argument of the APL+Win native event handler
            }
        }

        #region APL+Win ActiveX 'Native' Methods and Properties
        public object Application
        {
            get { return wsEngine; }
        }
        public object Call(string FunctionName, object Argument, object LeftArgument)
        {
            return wsEngine.Call(FunctionName, Argument, LeftArgument);
        }
        public object Call0(string FunctionName)
        {
            return wsEngine.Call0(FunctionName);
        }
        public object Call1(string FunctionName, object Argument)
        {
            return wsEngine.Call1(FunctionName, Argument);
        }
        public void CanClose()
        {
            wsEngine.CanClose();
        }
        public void Close()
        {
            wsEngine.CanClose();
        }
        public object CreateObject(string ObjectClass)
        {
            return wsEngine.CreateObject(ObjectClass);
        }
        public object Exec(string Expression)
        {
            return wsEngine.Exec(Expression);
        }
        public int SetOrphanTimeout(int Timeout, int ParentProcessId)
        {
            return wsEngine.SetOrphanTimeout(Timeout, ParentProcessId);
        }
        public object SysCall(string FunctionName, object Argument, object LeftArgument)
        {
            return wsEngine.SysCall(FunctionName, Argument, LeftArgument);
        }
        public object SysCall0(string FunctionName)
        {
            return wsEngine.SysCall0(FunctionName);
        }
        public object SysCall1(string FunctionName, object Argument)
        {
            return wsEngine.SysCall1(FunctionName, Argument);
        }
        public void SysCommand(string Command)
        {
            wsEngine.SysCommand(Command);
        }
        public int Visible
        {
            get { return wsEngine.Visible; }
            set { wsEngine.Visible = value; }
        }
        public object get_SysVariable(string VariableName)
        {
            return wsEngine.get_SysVariable(VariableName);
        }
        public object get_Variable(string VariableName)
        {
            return wsEngine.get_Variable(VariableName);
        }
        public void set_SysVariable(string VariableName, object pValue)
        {
            wsEngine.set_SysVariable(VariableName, pValue);
        }
        public void set_Variable(string VariableName, object pResult)
        {
            wsEngine.set_Variable(VariableName, pResult);
        }
        #endregion

#region Additional Methods and Properties with 'friendly' names and overloads eliminating 'missing' arguments
        public Int32 APLIndexOrigin
        {
            get { return (Int32) wsEngine.get_SysVariable("IO"); }
            set { wsEngine.set_SysVariable("IO", value); }
        }
        public void APLWorkspaceLoad(string wsPath)
        {
            if (System.IO.File.Exists(wsPath))
            {
                wsEngine.SysCommand("LOAD " + wsPath.Substring(0,wsPath.Length-3));
                //^Remove the ".w3" suffix
            }
            else
            {
                throw new Exception("APL+Win workspace not accessible: "+wsPath);
            }
        }

#endregion
    }
    public class APLNotifyEventArgs : EventArgs
    {
        public APLNotifyEventArgs(object code, object data, object result)
        {
            Code = code;
            Data = data;
            Result = result;
        }
        public object Code { get; set; }
        public object Data { get; set; }
        public object Result { get; set; }
    }
    public class APLSysNotifyEventArgs : EventArgs
    {
        public APLSysNotifyEventArgs(object code, object data, object result)
        {
            Code = code;
            Data = data;
            Result = result;
        }
        public object Code { get; set; }
        public object Data { get; set; }
        public object Result { get; set; }
    }
}
